//! Debug.h

//! Debug defines 
/*

	\author	Nuno Pereira
	\date Act 2005.
*/

#ifndef _DEBUG_H_
#define _DEBUG_H_

// The compiler defines _DEBUG when you specify the /MTd or /Mdd option... 
#ifdef _DEBUG	/* For DEBUG builds */		

#include <stdio.h>
#include <limits.h>
#include <omnetpp.h>

//
// Debug Level and Mask definitions.
//
enum DBG_LEVEL {
	dbgLevelDeath=1,
	dbgLevelCriticalError,
	dbgLevelFailure,
	dbgLevelInfo,
	dbgLevelTrace,
	dbgLevelVerbose
};

enum DBG_MASK {
	dbgMaskInit=1,
	dbgMaskMACMsgs=2,
	dbgMaskMACClock=4,
	dbgMaskMACProtocol=8, 
	dbgMaskRadio=16,
	dbgMaskStream=32,
	dbgMaskStat=64,
	dbgMaskAll=UINT_MAX
};

const int dbgMaskMAC=dbgMaskMACMsgs+dbgMaskMACClock+dbgMaskMACProtocol;
//
// Debug trace level and debug trace mask
//
//#define DbgTraceLevel	dbgLevelInfo
//#define DbgTraceMask	dbgMaskAll
//#define DbgTraceLevel	dbgLevelVerbose
//#define DbgTraceMask	dbgMaskAll
//#define DbgTraceLevel	dbgLevelTrace
#define DbgTraceLevel	dbgLevelFailure
#define DbgTraceMask	dbgMaskMACProtocol //+ dbgMaskRadio //+dbgMaskMACClock
//#define DbgTraceLevel	dbgLevelTrace
//#define DbgTraceMask	dbgMaskRadio+dbgMaskMACProtocol
//#define DbgTraceLevel	dbgLevelTrace
//#define DbgTraceMask	dbgMaskMAC

inline void DbgOut(int _DebugLevel, int _DebugMask, char *_OutFrmt, ...) {   			
    if ((DbgTraceLevel >= _DebugLevel) && ((_DebugMask) & DbgTraceMask)) {            		
		va_list args;
        va_start(args, _OutFrmt);
        vfprintf(stdout, _OutFrmt, args); // best for debugging, no buffered output...
//		ev.printf(_OutFrmt, args); // in omnet, more convenient?...
        va_end(args);
    }                                               		
}

#else /* ifdef _DEBUG; For RELEASE builds */

inline void DbgOut(int _DebugLevel, int _DebugMask, char *_OutFrmt, ...) {   			
}

#endif /* ifdef _DEBUG */

//
// bubbles are a special case of debug...
#define _BUBBLES

#ifdef _BUBBLES 

inline void BUBBLE(cModule *module, char *_OutFrmt, ...) {
	char msg[250];
	va_list args;
    va_start(args, _OutFrmt);
    vsprintf(msg, _OutFrmt, args);
	va_end(args); 	
	// this assumes that it called from a Node submodule...
	cModule *node = module;
	while (strstr(node->fullName(), "node") == NULL && node->parentModule() != NULL) {
		node=node->parentModule();
	}
	if (ev.isGUI() && strstr(node->fullName(), "node") != NULL) {
		module->bubble(msg);
		node->bubble(msg);
	}
}

#else

inline BUBBLE(char *_OutFrmt, ...) {
}

#endif //ifdef _BUBBLES 

#endif /* ifdef _DEBUG_H_ */
