//! Observer.cpp

#include <omnetpp.h>
#include <assert.h>
#include "Observer.h"

Define_Module( Observer );

//!Do initialization of class data members and any other initialization procedure necessary
void Observer::initialize()
{
	coll=pError=trnmt=FALSE;
	prioErrorCounter=collisionCounter=messageTxCounter=tournamentCounter=tournamentErrorCounter=0;
	prioErrorCounterMultiplicity=collisionCounterMultiplicity=messageTxCounterMultiplicity=0;
	
	prioErrorVector.setName("prioritization errors");
	collisionVector.setName("collisions");
	messageTxVector.setName("Message transmissions");
	
	WATCH(tournamentCounter);
}

//!	Process messages received.
/*!

	\param msg - cMessage* : Message received.
*/
void Observer::handleMessage(cMessage *msg)
{
}

void Observer::collision()
{
	coll=TRUE;
	collisionCounter++;
	collisionCounterMultiplicity++;

}

void Observer::prioError()
{
	pError=TRUE;
	prioErrorCounter++;
	prioErrorCounterMultiplicity++;
}

void Observer::messageTx(int id)
{
//  printf ("messageTx(%d):: N%02d\n", tournamentCounter, id);
	messageTxCounter++;
	messageTxCounterMultiplicity++;
}

void Observer::tournamentStart()
{
	trnmt=TRUE;
}

void Observer::tournamentEnd(int id , int result)
{

	if (trnmt==TRUE) {
		checkTournament();
		coll=pError=trnmt=FALSE;
	}
/*
  if (result==0)
    printf ("tournamentEnd::TRNMT=%d  Node=%02d  Result=LOST\n", tournamentCounter, id);
  else if (result==1)
    printf ("tournamentEnd::TRNMT=%d  Node=%02d  Result=WON\n", tournamentCounter, id);
  else if (result==-1)
    printf ("tournamentEnd::TRNMT=%d  Node=%02d  Result=LSTNG\n", tournamentCounter, id);    
*/    	
}

void Observer::checkTournament() 
{
	tournamentCounter++;
	collisionVector.record(collisionCounterMultiplicity);
	prioErrorVector.record(prioErrorCounterMultiplicity);
	messageTxVector.record(messageTxCounterMultiplicity);
	prioErrorCounterMultiplicity=collisionCounterMultiplicity=messageTxCounterMultiplicity=0;
	if (coll==TRUE || pError==TRUE) tournamentErrorCounter++;
}

//! Saves summary results (if any)
void Observer::finish()
{
    // save summary results (if any)
    ev << "duration (s): " << simTime() << endl;
    ev << "total transmitted messages: " << messageTxCounter << endl;
    ev << "total tournaments: " << tournamentCounter << endl;
    ev << "error tournaments: " << tournamentErrorCounter << endl;
    ev << "prioritization errors: " << prioErrorCounter << endl;
    ev << "collisions: " << collisionCounter << endl;

    recordScalar("duration (s)", simTime());
    recordScalar("total transmitted messages: ", messageTxCounter);
    recordScalar("total tournaments: ", tournamentCounter);
    recordScalar("error tournaments: ", tournamentErrorCounter);
    recordScalar("prioritization errors: ", prioErrorCounter);
    recordScalar("collisions: ", collisionCounter);    
}

Observer::~Observer()
{
}
